@@ -40,7 +40,6 @@ def terminate_if_status_is_not_available(
 ):
     prefix = f"[{provisioned_product_name}] {account_id}:{region}"
     logger.info(f"{prefix} :: checking if should be terminated")
-
     response = service_catalog.scan_provisioned_products_single_page(
         AccessLevelFilter={
             'Key': 'Account',
@@ -159,7 +158,7 @@ def provision_product_with_plan(
         ],
         NotificationArns=[
             regional_sns_topic,
-        ] if str(should_use_sns).lower() == 'true' else [],
+        ] if should_use_sns else [],
     )
     logger.info(f"{uid} :: Plan created, waiting for completion")
 
@@ -273,7 +272,7 @@ def provision_product(
         ],
         NotificationArns=[
             f"arn:aws:sns:{region}:{puppet_account_id}:servicecatalog-puppet-cloudformation-regional-events",
-        ] if str(should_use_sns).lower() == 'true' else [],
+        ] if should_use_sns else [],
     ).get('RecordDetail').get('ProvisionedProductId')
     logger.info(f"{uid}: provisioning started: {provisioned_product_id}")
 
@@ -552,21 +551,12 @@ def get_version_id_for(servicecatalog, product_id, version_name):
 def get_portfolio_for(servicecatalog, portfolio_name):
     result = None
 
-    response = servicecatalog.list_accepted_portfolio_shares(PortfolioShareType="AWS_ORGANIZATIONS")
-    assert response.get('NextPageToken') is None, "Pagination not supported"
-    for portfolio_detail in response.get('PortfolioDetails'):
-        if portfolio_detail.get('DisplayName') == portfolio_name:
-            result = portfolio_detail
-            break
-    logger.info(f"Did not find {portfolio_name} in AWS_ORGANIZATIONS PortfolioShareType")
-
     response = servicecatalog.list_accepted_portfolio_shares()
     assert response.get('NextPageToken') is None, "Pagination not supported"
     for portfolio_detail in response.get('PortfolioDetails'):
         if portfolio_detail.get('DisplayName') == portfolio_name:
             result = portfolio_detail
             break
-    logger.info(f"Did not find {portfolio_name} in IMPORTED PortfolioShareType")
 
     if result is None:
         response = servicecatalog.list_portfolios_single_page()
@@ -574,7 +564,6 @@ def get_portfolio_for(servicecatalog, portfolio_name):
             if portfolio_detail.get('DisplayName') == portfolio_name:
                 result = portfolio_detail
                 break
-    logger.info(f"Did not find {portfolio_name} in local portfolio list")
 
     assert result is not None, "Could not find portfolio"
     return result
