@@ -15,12 +15,6 @@ def cli(info, info_line_numbers):
     core.cli(info, info_line_numbers)
 
 
-@cli.command()
-@click.argument('f', type=click.File())
-def generate_shares(f):
-    core.generate_shares(f)
-
-
 @cli.command()
 @click.argument('f', type=click.File())
 @click.option('--single-account', default=None)
