@@ -261,39 +261,36 @@ def run_tasks_for_generate_shares(tasks_to_run):
             'accounts': [],
             'organizations': [],
         }
+        logger.info(f"generating policies collection for region {region}")
+        if os.path.exists(os.path.sep.join(['data', 'bucket'])):
+            logger.info(f"Updating policies for the region: {region}")
+            path = os.path.sep.join(['data', 'bucket', region, 'accounts'])
+            if os.path.exists(path):
+                for account_file in os.listdir(path):
+                    account = account_file.split(".")[0]
+                    sharing_policies['accounts'].append(account)
+
+            path = os.path.sep.join(['data', 'bucket', region, 'organizations'])
+            if os.path.exists(path):
+                for organization_file in os.listdir(path):
+                    organization = organization_file.split(".")[0]
+                    if organization != "":
+                        sharing_policies['organizations'].append(organization)
+
+        logger.info(f"Finished generating policies collection")
+
+        template = config.env.get_template('policies.template.yaml.j2').render(
+            sharing_policies=sharing_policies,
+            VERSION=version,
+        )
         with betterboto_client.ClientContextManager('cloudformation', region_name=region) as cloudformation:
-            cloudformation.ensure_deleted(StackName="servicecatalog-puppet-shares")
-
-            logger.info(f"generating policies collection for region {region}")
-            if os.path.exists(os.path.sep.join(['data', 'bucket'])):
-                logger.info(f"Updating policies for the region: {region}")
-                path = os.path.sep.join(['data', 'bucket', region, 'accounts'])
-                if os.path.exists(path):
-                    for account_file in os.listdir(path):
-                        account = account_file.split(".")[0]
-                        sharing_policies['accounts'].append(account)
-
-                path = os.path.sep.join(['data', 'bucket', region, 'organizations'])
-                if os.path.exists(path):
-                    for organization_file in os.listdir(path):
-                        organization = organization_file.split(".")[0]
-                        if organization != "":
-                            sharing_policies['organizations'].append(organization)
-
-            logger.info(f"Finished generating policies collection")
-
-            template = config.env.get_template('policies.template.yaml.j2').render(
-                sharing_policies=sharing_policies,
-                VERSION=version,
+            cloudformation.create_or_update(
+                StackName="servicecatalog-puppet-policies",
+                TemplateBody=template,
+                NotificationARNs=[
+                    f"arn:aws:sns:{region}:{puppet_account_id}:servicecatalog-puppet-cloudformation-regional-events"
+                ] if should_use_sns else [],
             )
-            with betterboto_client.ClientContextManager('cloudformation', region_name=region) as cloudformation:
-                cloudformation.create_or_update(
-                    StackName="servicecatalog-puppet-policies",
-                    TemplateBody=template,
-                    NotificationARNs=[
-                        f"arn:aws:sns:{region}:{puppet_account_id}:servicecatalog-puppet-cloudformation-regional-events"
-                    ] if str(should_use_sns).lower() == 'true' else [],
-                )
 
     for filename in glob('results/failure/*.json'):
         result = json.loads(open(filename, 'r').read())
